% cost and entry cost parameter values, for which data is to be analyzed
% (all relevant data must be first created and saved by programs CreatePolicies and
% CreateDensities)
costvector=[0.01 0.03 0.05 0.1 0.2 0.3 0.4 0.5]; % cost parameter values
phivector=[0.1 0.3 0.5 0.7 1]; % entry cost parameter values

poldir='policies\'; % Directory where optimal policies are saved
resultdir='densities\'; % Directory where industry equilibrium densities are saved
datadir='summarydata/'; % save summary data of results to this directory


% This program will draw contour plots in (phi,c) space for calibration
% targets. Intersection of these curves is where calibration targets are met. 
% If costvector and phivector are very sparse (as in this example), then the
% intersection is very approximate since these plots are based on
% simple interpolating in (phi,c) space.
turnovertarget=0.07; % Target for turnover
recaptarget=0.175; % Target for recapitalization rate

zinit=1; % This is needed for computing value at birth



% Draw all policies:
for costind=1:length(costvector),
    xgrid=load([poldir,num2str(costvector(costind)),'xvector']);
    ygrid=exp(xgrid); % 'x' in this program refers to log(revenue), and 'y' refers to revenue (i.e. corresponds to 'x' in the paper)
    sgrid=load([poldir,num2str(costvector(costind)),'svector']);
    pol=load([poldir,num2str(costvector(costind)),'polmatrix']);
    figure;pcolor(ygrid,sgrid,pol);title(['cost = ',num2str(costvector(costind))]);xlabel('revenue');ylabel('cash holdings');
end



turnovermatrix=zeros(length(costvector),length(phivector));
recapmatrix=zeros(length(costvector),length(phivector));
pricematrix=zeros(length(costvector),length(phivector));
pricematrix_unc=zeros(length(costvector),length(phivector));
turnovermatrix_unc=zeros(length(costvector),length(phivector));
zmatrix=zeros(length(costvector),length(phivector));
zmatrix_unc=zeros(length(costvector),length(phivector));
valmedmatrix=zeros(length(costvector),length(phivector)); % median value
valnewmatrix=zeros(length(costvector),length(phivector)); % value at birth



for costind=1:length(costvector),
    for phiind=1:length(phivector),
        turnovermatrix(costind,phiind)=load([resultdir,num2str(costvector(costind)),'turnover',num2str(phivector(phiind))]);
        recapmatrix(costind,phiind)=load([resultdir,num2str(costvector(costind)),'issuerate',num2str(phivector(phiind))]);
        pricematrix(costind,phiind)=load([resultdir,num2str(costvector(costind)),'price',num2str(phivector(phiind))]);
        pricematrix_unc(costind,phiind)=load([resultdir,num2str(costvector(costind)),'price_unc',num2str(phivector(phiind))]);
        turnovermatrix_unc(costind,phiind)=load([resultdir,num2str(costvector(costind)),'turnover_unc',num2str(phivector(phiind))]);
        zmatrix(costind,phiind)=load([resultdir,num2str(costvector(costind)),'zmean',num2str(phivector(phiind))]);
        zmatrix_unc(costind,phiind)=load([resultdir,num2str(costvector(costind)),'zmean_unc',num2str(phivector(phiind))]);
        
        %{
        %median value:
        if pricematrix(costind,phiind)>0,
            den=load([resultdir,num2str(costvector(costind)),'density',num2str(phivector(phiind))]);
            xgrid=load([poldir,num2str(costvector(costind)),'xvector']);
            sgrid=load([poldir,num2str(costvector(costind)),'svector']);
            V=load([poldir,num2str(costvector(costind)),'valuematrix']);
            pol=load([poldir,num2str(costvector(costind)),'polmatrix']);
            params=load([poldir,num2str(costvector(costind)),'params']);
            [Tl,Td,pol,xgrid,sgrid,V]=MakeTrans(params,pol,xgrid,sgrid,V);
            denvec=zeros(1,length(sgrid)*length(xgrid));
            Vvec=zeros(1,length(sgrid)*length(xgrid));
            for sind=1:length(sgrid),
                denvec((sind-1)*length(xgrid)+1:sind*length(xgrid))=den(sind,:);
                Vvec((sind-1)*length(xgrid)+1:sind*length(xgrid))=V(sind,:);
            end
            [Vsort,aa]=sort(Vvec);
            densorttemp=Vsort;
            densort=Vsort;
            for i=1:length(densort),
                densorttemp(i)=denvec(aa(i));
                densort(i)=sum(densorttemp(1:i));
            end
            ksi=params(7);gamma=params(8);
            medind=find(densort>0.5,1,'first');
            valmed=Vsort(medind);
            valmedmatrix(costind,phiind)=Vsort(medind);
            [BirthVector price]=FindBirth(params,phivector(phiind),zinit,pol,xgrid,sgrid,V);
            valnewmatrix(costind,phiind)=BirthVector*Vvec';
        else
            valmedmatrix(costind,phiind)=0;
            valnewmatrix(costind,phiind)=0;
        end
        %}
        
        
    end
end

figure;surf(phivector,costvector,turnovermatrix);title('turnover');xlabel('phi');ylabel('cost');
figure;surf(phivector,costvector,turnovermatrix./turnovermatrix_unc);title('turnover/turnover unconstrained');xlabel('phi');ylabel('cost');
figure;surf(phivector,costvector,recapmatrix);title('recapitalization rate');xlabel('phi');ylabel('cost');
%figure;surf(phivector,costvector,pricematrix);title('price');xlabel('phi');ylabel('cost');
figure;surf(phivector,costvector,pricematrix./pricematrix_unc);title('price/price unconstrained');xlabel('phi');ylabel('cost');
figure;surf(phivector,costvector,1./zmatrix);title('z newborn/z mean');xlabel('phi');ylabel('cost');
figure;surf(phivector,costvector,zmatrix./zmatrix_unc);title('z mean/z mean unconstrained');xlabel('phi');ylabel('cost');
%figure;surf(phivector,costvector,valnewmatrix./valmedmatrix);title('value at birth/median value');xlabel('phi');ylabel('cost');
figure;hold on;contour(phivector,costvector,turnovermatrix,turnovertarget);contour(phivector,costvector,recapmatrix,recaptarget);xlabel('phi');ylabel('cost');
legend(['turnover target ',num2str(turnovertarget)],['recap target ',num2str(recaptarget)]);title('calibration targets');


% save data:
save([datadir,'phivector'],'phivector','-ascii');
save([datadir,'costvector'],'costvector','-ascii');
save([datadir,'turnover'],'turnovermatrix','-ascii');
save([datadir,'turnover_unc'],'turnovermatrix_unc','-ascii');
save([datadir,'recaprate'],'recapmatrix','-ascii');
save([datadir,'price'],'pricematrix','-ascii');
save([datadir,'price_unc'],'pricematrix_unc','-ascii');
save([datadir,'meanoutput'],'zmatrix','-ascii');
save([datadir,'meanoutput_unc'],'zmatrix_unc','-ascii');
