%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function computes the equilibium distribution of firms of the unconstrained case
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [price_unc zmean_unc turnover_unc density_unc zgrid_unc]=ComputeUnc(params,entrycost,zinit);



alpha=params(1);sigma=params(2);r=params(3);rho=params(4);lambda=params(5);
cost=params(6);
%zinit=1;


beta=1/2-alpha/(sigma^2)-sqrt((alpha/(sigma^2)-1/2)^2+2*(rho+lambda)/(sigma^2));
ystar=(beta/(beta-1))*((rho+lambda)-alpha)*(cost/(rho+lambda)); %unconstrained exit threshold (y~GBM(alpha,sigma))
xstar=log(ystar);

options=optimset('Display','iter');
xinit=fsolve(@(x)exp(x)/(rho+lambda-alpha)-cost/(rho+lambda)-(ystar/(beta*(rho+lambda-alpha)))*(exp(x)/ystar).^beta-entrycost,xstar+1,options);

price_unc=exp(xinit)/zinit;


gamma=-2*alpha+sigma^2;
eta=sqrt(8*lambda*sigma^2+gamma^2);

rmean=(exp(xinit+xinit*eta/sigma^2)-exp((xinit*(gamma+eta)+xstar*(-gamma+eta+2*sigma^2))/(2*sigma^2)))*lambda./((exp(xinit*eta/sigma^2)-exp((xstar*(-gamma+eta)+xinit*(gamma+eta))/(2*sigma^2)))*(lambda-alpha));
age_mean=(-exp((xstar+xinit*(eta+2*alpha)/sigma^2)/2)*eta+exp((xinit+xstar*(eta+2*alpha)/sigma^2)/2).*(eta+2*(xinit - xstar)*lambda))./((-exp((xstar+xinit*(eta+2*alpha)/sigma^2)/2)+exp((xinit+xstar*(eta+2*alpha)/sigma^2)/2))*eta*lambda);
zmean_unc=rmean./price_unc;


f0=(2*lambda/eta)*(exp(eta*xinit/sigma^2)-exp(eta*xstar/sigma^2))/(exp(eta*xinit/sigma^2)-exp(((eta-gamma)*xstar+(eta+gamma)*xinit)/(2*sigma^2)));


% make grid for density
xgrid_unc=[xstar:(xinit-xstar)/100:xstar+20*(xinit-xstar)];
density_unc=zeros(size(xgrid_unc));

for xind=1:101,
    density_unc(xind)=f0*exp(-(gamma+eta)*(xgrid_unc(xind)-xinit)/(2*sigma^2))*((exp(eta*xgrid_unc(xind)/sigma^2)-exp(eta*xstar/sigma^2))/(exp(eta*xinit/sigma^2)-exp(eta*xstar/sigma^2)));
end
for xind=102:length(xgrid_unc),
    density_unc(xind)=f0*exp(-(gamma+eta)*(xgrid_unc(xind)-xinit)/(2*sigma^2));
end

% Normalize density so that it sums to one
density_unc=density_unc./sum(density_unc);

zgrid_unc=exp(xgrid_unc)./price_unc;
turnover_unc=(1/2)*f0*eta*exp((xstar*(eta-gamma)+xinit*(eta+gamma))/(2*sigma^2))/(exp(eta*xinit/sigma^2)-exp(eta*xstar/sigma^2))+lambda;

