% Main parameters
r=0.0425;
rho=0.0525;
alpha=0.0075; % This corresponds to my in the paper!
sigma=0.15;
lambda=0.04;

costvector=[0.01 0.03 0.05 0.1 0.2 0.3 0.4 0.5]; % List in this vector those values of 'c' for which you want to compute policies

% Units for money are in millions of dollars

% Transaction cost parameters for new cash injections
% These are from Hennessy and Whited (2007) (all firms)
ksi=0.0598; %fixed recapitalization cost
gamma=1.091; %marginal recapitalization cost NOTE! THis is different from the notation of the paper. This corresponds to 1+gamma of the paper!

% Corresponding parameters from Gomes (2001)
%ksi=0.08;
%gamma=1.028;

% This is the directory where results are saved:
resultdir='policies/';  %If you want to store results in a different directory, insert its name here

% Policy iteration parameters
iternum=10000; % maximum number of iterations within a given grid
dvmax=0.001; % stops iteration if continuation region remains constant for "stopnum" successive iterations 
iterfine=3; % the number of times the grid is made finer
%iterfine=5; % To get more accurate policy, use this value
fineratio=2; % Divide dx and ds by this number to get a finer grid
divlumpratio=10^6; % Refers to the size of dividend payment



% Create policies
for costind=1:length(costvector),
    
    cost=costvector(costind);
    
    params=[alpha,sigma,r,rho,lambda,cost,ksi,gamma];

    %Gridparameters
    % This program creates an evenly spaced grid that includes x=0, and uses transition
    % probability p so that variable 'x' is a arithmetic BM with drift my=alpha-(sigma^2)/2. 
    % Variable y=exp(x) then follows GBM(alpha,sigma). Variable pi referes to
    % profit, that is pi=y-cost.
    
    % Note! 'alpha' in this program corresponds to 'my' in the paper, and 'my'
    % in this program corresponds to 'my-(sigma^2)/2' in the paper !!!

    xmax=3;
    smax=4;
    beta=1/2-alpha/(sigma^2)-sqrt((alpha/(sigma^2)-1/2)^2+2*(rho+lambda)/(sigma^2));
    ystar=(beta/(beta-1))*((rho+lambda)-alpha)*(cost/(rho+lambda)); %unconstrained exit threshold (note that in this program y~GBM(alpha,sigma))
    xmin=log(ystar)-0.1;
    dxinit=0.1;  % Initial grid spacing for x
%    dsinit=0.1;  % Initial grid spacing for s
    dsinit=2*cost*0.1;  % Initial grid spacing for s

    % This initialization uses unconstrained value function to start
    [xgridinit,sgridinit,Vinit]=makegridsalternative(dxinit,dsinit,xmin,xmax,smax,alpha,sigma,rho,lambda,cost);


    % File names for the results:
    alku=num2str(cost); % Prefix for the file names
    loppu=''; % Postfix for the file names

    save([resultdir,alku,'params',loppu],'params','-ascii');
    [params,xgrid,sgrid,V,pol]=liquidrun(r,rho,lambda,alpha,sigma,xgridinit,sgridinit,Vinit,divlumpratio,iternum,dvmax,iterfine,fineratio,resultdir,alku,loppu,ksi,gamma,cost);
    save([resultdir,alku,'valuematrix',loppu],'V','-ascii');
    save([resultdir,alku,'polmatrix',loppu],'pol','-ascii');
    save([resultdir,alku,'xvector',loppu],'xgrid','-ascii');
    save([resultdir,alku,'svector',loppu],'sgrid','-ascii');

    dt=((xgrid(2)-xgrid(1))/sigma)^2;
        
end

