function [state_density_m zmean smean turnover issuerate]=FindDensity(params,xgrid,sgrid,pol,Tl,Td,BirthVector,Price,iternum,density_tol);

nx=length(xgrid);
ns=length(sgrid);
den=ones(nx*ns,1)./(nx*ns); % initial density (uniform)

sigma=params(2);
dt=((xgrid(2)-xgrid(1))/sigma)^2;

% Transition matrix:
T=Tl+BirthVector'*Td;

k=0;
err_density=1;

while k<=iternum && err_density>density_tol
    k=k+1;
    den=T*den;


% rescale state_density so that it's the density of firms, that is, the masses sum up to 1 after each
% iternum/100 iterations, and test the max norm of change in
% state_density to see if the iteration can stop
    if mod(k+1,iternum/100)==0
        den=den/sum(den);
        den_old=den;
        den=T*den;
        k=k+1;
        err_density=max(abs(den-den_old));
        [k err_density density_tol]
    end
end

den=den/sum(den);

death=Td'.*den; % actual death distribution per period
turnover=Td*den/dt; % this is fraction of deaths per time unit

issuevector=zeros(size(xgrid));
issuevector(find(pol(1,:)==3))=1;
issuerate=issuevector*den(1:nx)/dt; % This is the recapitalization rate


% matrix representations for key vectors:
state_density_m=zeros(ns,nx);
death_m=zeros(ns,nx);
birth_m=zeros(ns,nx);
for i=1:ns
    state_density_m(i,:)=den((i-1)*nx+[1:nx]);
    death_m(i,:)=death((i-1)*nx+[1:nx]);
    birth_m(i,:)=BirthVector((i-1)*nx+[1:nx]);
end


%figure;
%surf(xgrid,sgrid,state_density_m.^0.1);
%hold on;


% compute the outcome, e.g. marginal distribution of x and s, mean profit,
% mean output, mean cash holdings
xmarginal=sum(state_density_m);
smarginal=sum(state_density_m');

%save([result_dir,alku,'xvector',loppu],'xgrid','-ascii');
%save([result_dir,alku,'svector',loppu],'sgrid','-ascii');
%save([result_dir,alku,'stateDensityMat',loppu],'state_density_m','-ascii');
%save([result_dir,alku,'xmarginal',loppu],'xmarginal','-ascii');
%save([result_dir,alku,'smarginal',loppu],'smarginal','-ascii');


alpha=params(1);sigma=params(2);r=params(3);rho=params(4);lambda=params(5);
ksi=sigma^2-2*alpha;
ny=sqrt(8*lambda*sigma^2+ksi^2);
tau=(ksi+ny)/(2*sigma^2); % (exponential parameter for tail distribution)

tailmass=xmarginal(end); % mass in tail
xmarginal(end)=0;
rmean=sum(exp(xgrid).*xmarginal)+tailmass*(tau/(tau-1))*exp(xgrid(end)); %mean revenue
zmean=rmean/Price;
smean=sum(sgrid.*smarginal);
