function [Tl,Td,pol,xgrid,sgrid,V,BirthVector,Price]=MakeTransAllowTail(params,pol,xgrid,sgrid,V,entrycost,zinit);


% This version 1) makes transition matrixes for computing equilibrium density, and 2) finds birth point.
% Contrary to an earlier version, this version works also when birth point is in "tail" (tail=policy region where no cash is
% held)

% NOte: we have two 'ksi's, so:
% call the fixed recapitalization cost 'ksicost' to distinguish from the other 'ksi'

% This function computes the birth vector and price assuming that one unit mass is born
% and split between two x-values so that expected value equal to entrycost

% zinit is the output at birth. In effect, firms are born at two output
% levels, zminus and zplus, zminus<zinit<zplus, such that expected
% output at birth is equal to zinit, and each firm's expected value at birth is 
% exactly equal to entry cost. This is done in order to ensure that
% all new firms hit a grid point.

% In this version, it is assumed that fixed cost of entry ("entrycost")
% includes only the "physical" cost. The total cost of entry is then
% gamma*entrycost+ksicost+gamma*sinit, where sinit is endogenously determined

% Free-entry condition is hence:
% V(Price*zinit,sinit)=gamma*entrycost+ksicost+gamma*sinit


ksicost=params(7);
gamma=params(8);

alpha=params(1);sigma=params(2);r=params(3);rho=params(4);lambda=params(5);
cost=params(6);

if alpha>=lambda
    error('The condition alpha<lambda is not satisfied!');
end

% NOTE: here alpha is drift of GBM, and my is corresponding drift of ABM!

dx=xgrid(2)-xgrid(1);
ds=sgrid(2)-sgrid(1);
my=alpha-(sigma^2)/2;
dt=(dx/sigma)^2;
p=(1/2)*(1+(my/sigma)*sqrt(dt));
q=1-p;
beta=1/2-alpha/(sigma^2)-sqrt((alpha/(sigma^2)-1/2)^2+2*(rho+lambda)/(sigma^2));

% f=f0 exp(tau (x-x0)) in the uncontrained case in the tail
%tau1=alpha/sigma^2-1/2-sqrt((alpha-sigma^2/2)^2+2*lambda*sigma^2)/sigma^2
ksi=sigma^2-2*alpha;
ny=sqrt(8*lambda*sigma^2+ksi^2);
tau=(ksi+ny)/(2*sigma^2);



%% take the rectangle cutting the continuation region as our new grid
% right boundary
xmaxind=find(pol(1,:)==2,1,'first');
xgrid=xgrid(1:xmaxind);
pol=pol(:,1:xmaxind);
V=V(:,1:xmaxind);

% left boundary
i=1;
while pol(:,i)==0
    i=i+1;
end
xminind=i;
xgrid=xgrid(xminind:end);
pol=pol(:,xminind:end);
V=V(:,xminind:end);
nx=length(xgrid);
% index of the point x=0
xzeroind=find(xgrid==0);


% upper boundary
[i,j]=find(pol==1);
smaxind=min(length(sgrid),max(i)+1);
%smaxind=max(i)+1;
sgrid=sgrid(1:smaxind);
pol=pol(1:smaxind,:);
V=V(1:smaxind,:);
ns=smaxind;


XN=length(xgrid);
SN=length(sgrid);
    

% FInd birth point, and extend grid to the right if necessary:
valuevector=zeros(1,XN);
targetvector=zeros(1,XN);
for xind=1:XN,
    dif=V(:,xind)-sgrid'*gamma;
    valuevector(xind)=max(dif);
    targetvector(xind)=find(dif==max(dif),1,'first');
end

if (gamma*entrycost+ksicost)<valuevector(end),
    BirthVector=zeros(1,XN*SN);
    R=find(valuevector>(gamma*entrycost+ksicost),1,'first');
    probR=((gamma*entrycost+ksicost)-valuevector(R-1))/(valuevector(R)-valuevector(R-1));
    BirthVector((targetvector(R-1)-1)*XN+R-1)=1-probR;
    BirthVector((targetvector(R)-1)*XN+R)=probR;
    Price=(probR*exp(xgrid(R))+(1-probR)*exp(xgrid(R-1)))/zinit;    
else  % Birth point in the tail
%    ystar=(beta/(beta-1))*((rho+lambda)-alpha)*(cost/(rho+lambda)); %unconstrained exit threshold (y~GBM(alpha,sigma))
    
    A=(V(1,XN)-exp(xgrid(XN))/((rho+lambda)-alpha)+cost/(rho+lambda))/(exp(xgrid(XN))^beta);
  
    stopindex=0;xind=XN+1;
    
    while stopindex==0,
        xgrid=[xgrid xgrid(end)+dx];
        pol=[pol 2.*ones(SN,1)];
        V=[V zeros(SN,1)];
        V(1,xind)=exp(xgrid(xind))/((rho+lambda)-alpha)-cost/(rho+lambda)+A*exp(xgrid(xind))^beta;
        V(:,xind)=V(1,xind)+sgrid';
        if (gamma*entrycost+ksicost)<V(1,xind),
            BirthVector=zeros(1,(xind+1)*SN);
            probR=((gamma*entrycost+ksicost)-V(1,xind-1))/(V(1,xind)-V(1,xind-1));
            BirthVector(xind-1)=1-probR;
            BirthVector(xind)=probR;
            Price=(probR*exp(xgrid(xind))+(1-probR)*exp(xgrid(xind-1)))/zinit;
            stopindex=1;
        end
        xind=xind+1;
    end
    xgrid=[xgrid xgrid(end)+dx];
    pol=[pol 2.*ones(SN,1)];
    V=[V zeros(SN,1)];
    V(1,xind)=exp(xgrid(xind))/((rho+lambda)-alpha)-cost/(rho+lambda)+A*exp(xgrid(xind))^beta;
    V(:,xind)=V(1,xind)+sgrid';
end
XN=length(xgrid);
nx=length(xgrid);


%% build the transition matrix of distrition Tl and death rate vector Td so that the continuing firms 
%% with mass distribution f will have mass distribution Tl*f after one time step, and Td*f mass of firms 
%% will be die and re-born at the birth point. Tl(i,j) is the probability of a firm at state j to live and jump to 
%% state i; Td(i) is the probability of a firm at state j to die.



Tl=sparse(ns*nx,ns*nx);
Td=sparse(1,ns*nx);




for sind=1:ns,
    for xind=1:nx-1,
        
        % exit
        if pol(sind,xind)==0,
            Td((sind-1)*nx+xind)=1; %death (exit region)
        end
        
        %continuation
        if (pol(sind,xind)==1 || pol(sind,xind)==4), % continuation region
            Td((sind-1)*nx+xind)=lambda*dt; % death
            piv=exp(xgrid(xind))-cost;
            snew=sgrid(sind)*(1+r*dt)+piv*dt;
            sindlow=floor(snew/ds)+1; % index for lower destination s-point
            plow=(snew-(sindlow-1)*ds)/ds; % probability of hitting lower s-point
            
            % if jump below s=0 line, then assume that jumps to s=0 with
            % probability one
            if sindlow<1,
                sindlow=1;plow=1;
            end
            % if jump to s>smax, then assume that jumps to s=smax with
            % probability one
            if sindlow>=ns,
                sindlow=ns-1;plow=0;
            end
            
            if xind==1,
                Td((sind-1)*nx+xind)=Td((sind-1)*nx+xind)+(1-lambda*dt)*q;
                Tl((sindlow-1)*nx+xind+1,(sind-1)*nx+xind)=(1-lambda*dt)*plow*p;
                Tl(sindlow*nx+xind+1,(sind-1)*nx+xind)=(1-lambda*dt)*(1-plow)*p;
            else
   
                
                Tl((sindlow-1)*nx+xind-1,(sind-1)*nx+xind)=(1-lambda*dt)*plow*q;
                Tl((sindlow-1)*nx+xind+1,(sind-1)*nx+xind)=(1-lambda*dt)*plow*p;
                Tl(sindlow*nx+xind-1,(sind-1)*nx+xind)=(1-lambda*dt)*(1-plow)*q;
                Tl(sindlow*nx+xind+1,(sind-1)*nx+xind)=(1-lambda*dt)*(1-plow)*p;            
            end
        end
        
        if pol(sind,xind)==2,
            if pol(1,xind)==2,
           %     if pol(sind,xind)==2,
                Td((sind-1)*nx+xind)=lambda*dt; % death
                Tl(xind-1,(sind-1)*nx+xind)=(1-lambda*dt)*q;
                Tl(xind+1,(sind-1)*nx+xind)=(1-lambda*dt)*p;
            else
                starget=max(1,find(pol(:,xind)==1,1,'last'));
                Tl(:,(sind-1)*nx+xind)=Tl(:,(starget-1)*nx+xind);
                Td((sind-1)*nx+xind)=Td((starget-1)*nx+xind);
            end
        end
    end
end

% lowest row re-capitalization and dividends:
sind=1;
for xind=1:nx-1,
    
    %{
    if pol(sind,xind)==0,
        Td((sind-1)*nx+xind)=1; %death (exit region)
    end
    %}
    if pol(sind,xind)==3,
        starget=find(pol(:,xind)==4,1,'first');
        Tl(:,(sind-1)*nx+xind)=Tl(:,(starget-1)*nx+xind);
        Td((sind-1)*nx+xind)=Td((starget-1)*nx+xind);
    end
    %{
    if pol(sind,xind)==1,
        Td((sind-1)*nx+xind)=lambda*dt; % death
            piv=exp(xgrid(i))-cost;
            snew=sgrid(sind)*(1+r*dt)+piv*dt;
            sindlow=floor(snew/ds)+1; % index for lower destination s-point
            plow=(snew-(sindlow-1)*ds)/ds; % probability of hitting lower s-point
            
            % if jump below s=0 line, then assume that jumps to s=0 with
            % probability one
            if sindlow<1,
                sindlow=1;plow=1;
            end
            % if jump to s>smax, then assume that jumps to s=smax with
            % probability one
            if sindlow>=ns,
                sindlow=ns-1;plow=0;
            end
            
            if xind==1,
                Td((sind-1)*nx+xind)=Td((sind-1)*nx+xind)+(1-lambda*dt)*q;
            else
                Tl((sindlow-1)*nx+xind-1,(sind-1)*nx+xind)=(1-lambda*dt)*plow*q;
                Tl((sindlow-1)*nx+xind+1,(sind-1)*nx+xind)=(1-lambda*dt)*plow*p;
                Tl(sindlow*nx+xind-1,(sind-1)*nx+xind)=(1-lambda*dt)*(1-plow)*q;
                Tl(sindlow*nx+xind+1,(sind-1)*nx+xind)=(1-lambda*dt)*(1-plow)*p;            
            end
        end
        %}
    
        if pol(sind,xind)==2,
            Td((sind-1)*nx+xind)=lambda*dt; % death
            Tl((sind-1)*nx+xind-1,(sind-1)*nx+xind)=(1-lambda*dt)*q;
            Tl((sind-1)*nx+xind+1,(sind-1)*nx+xind)=(1-lambda*dt)*p;
        end
end


% Tail:
% (This assumes that birth point is not in the tail)
Tl(nx-1,nx)=(1-exp(-tau*dx))*(1-lambda*dt)*q; % The fraction of tail within the leftmost dx is (1-exp(-tau*dx))
Td(nx)=lambda*dt; % death in tail
Tl(nx,nx)=1-Tl(nx-1,nx)-Td(nx); % The rest stays in tail

%Tl(nx-1,nx)=(1-lambda*dt)*q*exp(tau*dx); % move to left from tail
%Td(nx)=lambda*dt*exp(tau*dx)/(1-exp(tau*dx)); % death in tail
%Tl(nx,nx)=1-Tl(nx-1,nx)-Td(nx); % The rest stays in tail

for sind=2:ns,
    Tl(:,sind*nx)=Tl(:,nx);
    Td(sind*nx)=Td(nx);
end

