function [V,pol]=fun(r,rho,lambda,alpha,sigma,iternum,dvmax,divlump,xgrid,sgrid,Vinit,resultdir,alku,loppu,ksi,gamma,cost);
% This version runs vertically
% This version has death rate lambda
% This version takes xgrid, sgrid, and initial V as arguments
% The iteration stops when either 1) number of iterations reaches
% "iternum", or 2) norm(change in value matrix) remains below "dvmax" for
% 10 successive iterations


% This function takes an evenly spaced grid that includes x=0, and uses transition
% probability p so that variable 'x' is a arithmetic BM with drift my=alpha-(sigma^2)/2. 
% Variable y=exp(x) then follows GBM(alpha,sigma). Variable pi referes to
% profit, that is pi=y-cost.

saveinterval=50; %number of periods between saving new results
xgridprev=xgrid;sgridprev=sgrid;Vprev=Vinit;polprev=zeros(size(Vinit));


% Initial saving to make sure that saved vecotors and matrixes are always
% the same size (even if program runs less than 50 rounds):
%save([resultdir,alku,'valuematrixprev',loppu],'Vprev','-ascii');
%save([resultdir,alku,'polmatrixprev',loppu],'polprev','-ascii');
%save([resultdir,alku,'xvectorprev',loppu],'xgridprev','-ascii');
%save([resultdir,alku,'svectorprev',loppu],'sgridprev','-ascii');
%save([resultdir,alku,'valuematrix',loppu],'Vinit','-ascii');
%save([resultdir,alku,'polmatrix',loppu],'polprev','-ascii');
%save([resultdir,alku,'xvector',loppu],'xgrid','-ascii');
%save([resultdir,alku,'svector',loppu],'sgrid','-ascii');





dx=xgrid(2)-xgrid(1);ds=sgrid(2)-sgrid(1);

% Define my - it is used in calculating transition probabilities
my=alpha-(sigma^2)/2;
beta=1/2-alpha/(sigma^2)-sqrt((alpha/(sigma^2)-1/2)^2+2*(rho+lambda)/(sigma^2));



% Simply make new grids as follows:
pigrid=exp(xgrid)-cost;
ygrid=exp(xgrid);


V=Vinit;Vold=V;
dt=(dx/sigma)^2;
pol=zeros(size(V)); %policy function: 0=exit, 1=continue, 2=dividend, 3=raise capital, 4=optimal cash injection
A=0; %THis is just to initialize this in order to avoid an error if xmax is too small
p=(1/2)*(1+(my/sigma)*sqrt(dt));q=1-p;

XN=size(xgrid,2);
SN=size(sgrid,2);

% Calculate xzeroind in the simplest way:
xzeroind=1;
for xind=1:XN,
%    if xgrid(xind)<=0,xzeroind=xind;end
    if pigrid(xind)<=0,xzeroind=xind;end
end



%iteration

iterstop=0;iter=1;contnumold=0;contcount=0;savecount=1;

while iterstop==0,
    
polold=pol;
Vold=V;
pol=zeros(size(V));
contnum=0; % number of grid points within this iteration where optimal policy is to continue (i.e. size of continuation region)

% Boundaries of V:
V(:,1)=sgrid';
for xind=1:xzeroind,
    Vsvec=Vold(:,xind)-sgrid'*gamma-ksi;
    Vcand=max(Vsvec);
    Vsind=min(find(Vsvec==max(Vsvec)));
    Vcand2=max(interp1(sgrid,Vsvec',[sgrid(max(1,Vsind-1)):(sgrid(2)-sgrid(1))/10:sgrid(Vsind+1)],'cubic'));
    V(1,xind)=max(0,Vcand2);
    if V(1,xind)>0,
        pol(1,xind)=3;
        pol(Vsind,xind)=4;
    end
end
%V(1,1:xzeroind)=0;
divind=0; % Indicator variable for "dividend threshold has been reached"
piright=0; % initialize piright (the largest pi in continuation region for s=0)
for xind=xzeroind+1:XN-1,
    if divind==0,
        %continuation value:
        
        frtmp=pigrid(xind)*dt/ds;
        con=(1/(1+rho*dt))*(p*(Vold(1,xind+1).*(1-frtmp)+Vold(2,xind+1).*frtmp)+q*(Vold(1,xind-1).*(1-frtmp)+Vold(2,xind-1).*frtmp));       
%        con=(1/(1+rho*dt))*(p*interp1(sgrid',Vold(:,xind+1),pigrid(xind)*dt,'linear')+q*interp1(sgrid',Vold(:,xind-1),pigrid(xind)*dt,'linear'));
        con=(1-lambda*dt)*con; %correction due to death rate lambda
        %value with dividends:
        div=(1/(1+r*dt))*pigrid(xind)*dt+(1/(1+rho*dt))*(p*Vold(1,xind+1)+q*Vold(1,xind-1));
        div=(1-lambda*dt)*div; %correction due to death rate lambda
        V(1,xind)=max(con,div);
        if con>div,
            pol(1,xind)=1;
            contnum=contnum+1;
            piright=pigrid(xind); % the largest x in continuation region for s=0
        else        
            pol(:,xind)=2;
            beta=1/2-alpha/(sigma^2)-sqrt((alpha/(sigma^2)-1/2)^2+2*(rho+lambda)/(sigma^2));
            A=(div-ygrid(xind)/((rho+lambda)-alpha)+cost/(rho+lambda))/(ygrid(xind)^beta);
            divind=1;
            V(2:SN,xind)=sgrid(2:SN)'+div;
        end
    else  %dividend thershold has been reached
        V(1,xind)=ygrid(xind)/((rho+lambda)-alpha)-cost/(rho+lambda)+A*ygrid(xind)^beta;
        V(2:SN,xind)=sgrid(2:SN)'+V(1,xind);
        pol(:,xind)=2;
    end
end
% right hand side boundary is calculated separately (so that xgrid+1 does not result in error)
V(:,XN)=ygrid(XN)/((rho+lambda)-alpha)-cost/(rho+lambda)+A*ygrid(XN)^beta+sgrid';
pol(:,XN)=2;


% Interior of V:
contind=1; %indicator variable: zero if continuation region empty with the current s, otherwise one
%sind=2;
xind=2;
pileft=0;

while contind==1, % THis is the loop that runs x from 2 upwards until continuation region empty
    sind=2;
    while pol(sind-1,xind)==0 && sind<=SN,  %exit region
        snew=max(0,sgrid(sind)*(1+r*dt)+pigrid(xind)*dt);  % This is s+ds if no dividens will be payed
        
        indtmp=min(floor(snew/ds)+1,SN-1);
        frtmp=(snew-sgrid(indtmp))/ds;
        
        
        con=(1/(1+rho*dt))*(p*(Vold(indtmp,xind+1).*(1-frtmp)+Vold(indtmp+1,xind+1).*frtmp)+q*(Vold(indtmp,xind-1).*(1-frtmp)+Vold(indtmp+1,xind-1).*frtmp));
%        con=(1/(1+rho*dt))*(p*interp1(sgrid',Vold(:,xind+1),snew,'linear')+q*interp1(sgrid',Vold(:,xind-1),snew,'linear'));
        con=(1-lambda*dt)*con+lambda*dt*sgrid(sind); %correction due to death rate lambda
        if con<sgrid(sind),  % Stopping is optimal
                V(sind,xind)=sgrid(sind);
                pol(sind,xind)=0;
        else
            V(sind,xind)=con;
            pol(sind,xind)=1;
            contnum=contnum+1;
        end
        sind=sind+1;
    end
    
    while (pol(sind-1,xind)==1 || pol(sind-1,xind)==3 || pol(sind-1,xind)==4) && sind<=SN,   %continuation region
        snew=max(0,sgrid(sind)*(1+r*dt)+pigrid(xind)*dt);  % This is s+ds if no dividens will be payed
        sdivnew=max(0,(sgrid(sind)-divlump)*(1+r*dt)+pigrid(xind)*dt); % This is s+ds if dividends will be payed
        indtmp=min(floor(snew/ds)+1,SN-1);frtmp=(snew-sgrid(indtmp))/ds;
        inddivtmp=min(floor(sdivnew/ds)+1,SN-1);frdivtmp=(sdivnew-sgrid(inddivtmp))/ds;
        if sdivnew>0, divpay=divlump; else divpay=pigrid(xind)*dt/(1+r*dt)+sgrid(sind);end % Dividend payment
        indtmp=min(floor(snew/ds)+1,SN-1);frtmp=(snew-sgrid(indtmp))/ds;
        con=(1/(1+rho*dt))*(p*(Vold(indtmp,xind+1).*(1-frtmp)+Vold(indtmp+1,xind+1).*frtmp)+q*(Vold(indtmp,xind-1).*(1-frtmp)+Vold(indtmp+1,xind-1).*frtmp));        
  %      con=(1/(1+rho*dt))*(p*interp1(sgrid',Vold(:,xind+1),snew,'linear')+q*interp1(sgrid',Vold(:,xind-1),snew,'linear'));
        con=(1-lambda*dt)*con+lambda*dt*sgrid(sind); %correction due to death rate lambda
        div=divpay+(1/(1+rho*dt))*(p*(Vold(inddivtmp,xind+1).*(1-frdivtmp)+Vold(inddivtmp+1,xind+1).*frdivtmp)+q*(Vold(inddivtmp,xind-1).*(1-frdivtmp)+Vold(inddivtmp+1,xind-1).*frdivtmp));
%        div=divpay+(1/(1+rho*dt))*(p*interp1(sgrid',Vold(:,xind+1),sdivnew,'linear')+q*interp1(sgrid',Vold(:,xind-1),sdivnew,'linear'));
        div=(1-lambda*dt)*div+lambda*dt*sgrid(sind); %correction due to death rate lambda
        if con>div %continuation is optimal
            V(sind,xind)=con;
            if pol(sind,xind)<4,
                pol(sind,xind)=1;
            end
            contnum=contnum+1;
        else
            V(sind:SN,xind)=div+(sgrid(sind:SN)'-sgrid(sind));
            pol(sind:SN,xind)=2;
        end
        sind=sind+1;
    end
    xind=xind+1;
    if (pol(1,xind)==2 || xind==XN),contind=0;end
end


contsize=contnum*dx*ds; % The size of the continuation region
[iter norm(V-Vold) contsize]



% Save previous and current parameters:
%{
if savecount==saveinterval,
    save([resultdir,alku,'valuematrixprev',loppu],'Vprev','-ascii');
    save([resultdir,alku,'polmatrixprev',loppu],'polprev','-ascii');
    save([resultdir,alku,'xvectorprev',loppu],'xgridprev','-ascii');
    save([resultdir,alku,'svectorprev',loppu],'sgridprev','-ascii');
    xgridprev=xgrid;sgridprev=sgrid;Vprev=V;polprev=pol;
    save([resultdir,alku,'valuematrix',loppu],'V','-ascii');
    save([resultdir,alku,'polmatrix',loppu],'pol','-ascii');
    save([resultdir,alku,'xvector',loppu],'xgrid','-ascii');
    save([resultdir,alku,'svector',loppu],'sgrid','-ascii');
    savecount=1;
else
    savecount=savecount+1;
end
%}    

%save tempfile pigrid sgrid pol V;

% stop if number of iterations reaches iternum
if iter>=iternum,
    iterstop=1;
end
iter=iter+1;

%stop if norm(change in valuematrix) remains below dvmax for 10 successive
%iterations
if norm(V-Vold)<dvmax,
    contcount=contcount+1;
else
    contcount=1;
end
contnumold=contnum;
if contcount>=10,
    iterstop=1;
end


end;



%save([resultdir,alku,'valuematrix',loppu],'V','-ascii');
%save([resultdir,alku,'polmatrix',loppu],'pol','-ascii');
%save([resultdir,alku,'xvector',loppu],'xgrid','-ascii');
%save([resultdir,alku,'svector',loppu],'sgrid','-ascii');



% Make vectors smaller
%xx=XN;ss=SN;
%for xind=1:XN,
%    if pigrid(xind)<(1.5*piright),
%        xx=xind;
%    end
%end
%for sind=1:SN,
%    if sgrid(sind)<(1.5*sbar),
%        ss=sind;
%    end
%end

%xgrid=xgrid(1:xx);
%sgrid=sgrid(1:ss);
%pol=pol(1:ss,1:xx);
%V=V(1:ss,1:xx);
